﻿package  {
	import flash.events.Event;
	import flash.text.TextField;
	import flash.utils.getTimer;
	import org.papervision3d.core.math.Number3D;
	import org.papervision3d.core.proto.MaterialObject3D;
	import org.papervision3d.materials.special.CompositeMaterial;
	import org.papervision3d.materials.WireframeMaterial;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.objects.primitives.Sphere;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.render.QuadrantRenderEngine;
	import org.papervision3d.view.BasicView;
	import org.papervision3d.core.utils.MeshUtil;
	import org.papervision3d.core.math.Plane3D;
	
	/**
	 * 3D View
	 */
	public class World extends BasicView{
		private var sphere:Sphere;
		private var plane:Plane;
		private var mesh0;
		private var mesh1;
		/**
		 * コンストラクタ
		 */
		public function World() {
			//初期化
			init();
			//フレームレート表示
			showFPS();
		}

		public function init() {
			//------------------------
			//球体のマテリアル（カラー＋ワイヤーフレーム）
			var wireMat:WireframeMaterial = new WireframeMaterial(0x000000,0.5);
			var colorMat1:ColorMaterial = new ColorMaterial(0xFF0000, 1);
			colorMat1.doubleSided = true;
			var compMat1:CompositeMaterial = new CompositeMaterial();
			compMat1.addMaterial(wireMat);
			compMat1.addMaterial(colorMat1);
			//compMat1.doubleSided = true;
			//分割される前の球体
			sphere = new Sphere(compMat1, 100);
			//分割前のオブジェクト表示していると、表示がおかしくなるようなのでaddChildしない
			//scene.addChild(sphere);
			
			//------------------------
			//表示用平面のマテリアル（カラー＋ワイヤーフレーム）
			var colorMat2:ColorMaterial = new ColorMaterial(0x0000FF,0.4);
			colorMat2.doubleSided = true;
			var compMat2:CompositeMaterial = new CompositeMaterial();
			compMat2.addMaterial(wireMat);
			compMat2.addMaterial(colorMat2);
			compMat2.doubleSided = true;
			//表示用平面
			plane = new Plane(compMat2, 200, 200, 1, 1);
			scene.addChild(plane);

			//------------------------
			//カメラ位置調整
			camera.fov = 30;
			camera.y = 400;
			camera.z = -800;
			camera.target = sphere;
			//描画開始
			startRendering();
			//ポリゴン欠けないようにレンダラ変更
			//renderer = new QuadrantRenderEngine(QuadrantRenderEngine.ALL_FILTERS);

		}

		override protected function onRenderTick(event:Event = null):void {
			//表示用平面を回転させる
			plane.rotationX -= 2;
			plane.rotationY += 1;
			
			//表示用平面の3頂点を利用して、分割用平面の傾きを定義する
			var point1:Number3D = plane.geometry.vertices[0].getPosition();
			var point2:Number3D = plane.geometry.vertices[1].getPosition();
			var point3:Number3D = plane.geometry.vertices[2].getPosition();
			//頂点はローカル座標なので、平面の回転を加える
			point1.rotateX(plane.rotationX);
			point2.rotateX(plane.rotationX);
			point3.rotateX(plane.rotationX);
			point1.rotateY(plane.rotationY);
			point2.rotateY(plane.rotationY);
			point3.rotateY(plane.rotationY);
			point1.rotateZ(plane.rotationZ);
			point2.rotateZ(plane.rotationZ);
			point3.rotateZ(plane.rotationZ);
			//分割用平面
			var cuttingPlane:Plane3D = new Plane3D();
			cuttingPlane.setThreePoints(point1, point2, point3);
			//------------------------
			//メッシュ分割実行
			var meshes:Array = MeshUtil.cutTriangleMesh(sphere, cuttingPlane);
			//分割されたメッシュ
			if (mesh0 != null) {
				scene.removeChild(mesh0);
			}
			mesh0 = meshes[0];
			mesh0.x = 200;
			scene.addChild(mesh0);
			//分割されたメッシュ、もう片方
			if (mesh1 != null) {
				scene.removeChild(mesh1);
			}
			mesh1 = meshes[1];
			mesh1.x = -200;
			scene.addChild(mesh1);
			
			super.onRenderTick();
		}
		
		//---------------------------
		//以下、フレームレート計算用
		private	var fps_text:TextField = new TextField();
		private var update_time = 500;
		private	var draw_count = 0;
		private	var old_timer = getTimer();
		private function showFPS():void{
			fps_text.background = true;
			fps_text.selectable = false;
			fps_text.width = 70;
			fps_text.height = 18;
			this.addChild (fps_text);
			this.addEventListener(Event.ENTER_FRAME , onFPS);
		}
		private function onFPS(e:Event):void{
			draw_count += 1;
			if (getTimer()-old_timer >= update_time) {
				var fps = draw_count * 1000 / (getTimer() - old_timer);
				fps = Math.floor(fps * 10) / 10;  
				fps_text.text ="FPS:"+ fps + ' / ' + fps_text.stage.frameRate;
				old_timer = getTimer();
				draw_count = 0;
			}
		}
	}
}
